/*= Rohde & Schwarz ZV - Vector Network Analyzers ===========================*/
#include <utility.h>
#include <analysis.h>
#include <formatio.h>
#include <stdlib.h>
#include <userint.h>
#include "rszv.h"
#include "rszvu.h"

#define MAX_ARRAY 2001

/*= Global Variable Definitions =============================================*/
short uirCode;
long panelHandle[10];
unsigned long instrHandle;
ViStatus uirErr;

#define APPCHECKERR(fCal) if (uirErr = (fCal), uirErr < VI_SUCCESS) { \
                if (uirErr >= 0xBFFF0000) { \
                    abortApplication ();    \
                } else {                    \
                    SetWaitCursor(0);       \
                    rszv_error_message (VI_NULL, uirErr, errMsg);\
                    MessagePopup ("Error", errMsg);\
                    return 0;\
                }\
            } else
/*=================== Utility Function Declarations =========================*/
long abortApplication (void);

/*= Message Arrays ==========================================================*/
static char *initCtrlsHelpStr[] = {"\nThis control specifies if an ID Query is sent to the\n"
                                    "instrument during the initialization procedure.\n\n"
                                    "Valid Range:\n"
                                    "VI_FASLE (0) - Skip Query\n"
                                    "VI_TRUE  (1) - Do Query (Default Value)",
                                    
                                    "\nThis control specifies if the instrument is to be reset to its\n"
                                    "power-on settings during the initialization procedure.\n\n"
                                    "Valid Range:\n"
                                    "VI_FASLE (0) - Don't Reset\n"
                                    "VI_TRUE  (1) - Reset Device (Default Value)",
                                    
                                    "\nThis control specifies the interface and address of the device\n"
                                    "that is to be initialized (Instrument Descriptor). The exact\n"
                                    "grammar to be used in this control is shown in the note below.\n\n"
                                    "Default Value: GPIB::20\n\n"
                                    "Notes:\n\n"
                                    "(1) Based on the Instrument Descriptor, this operation\n"
                                    "establishes a communication session with a device.  The grammar\n"
                                    "for the Instrument Descriptor is shown below.  Optional\n"
                                    "parameters are shown in square brackets ([]).\n\n"
                                    "Interface   Grammar\n"
                                    "------------------------------------------------------\n"
                                    "Serial      ASRL<port>[::INSTR]\n"
                                    "GPIB        GPIB[board]::primary address[::secondary address]\n"
                                    "            [::INSTR]\n"
                                    "RSIB        RSIB::remote_host::INSTR\n\n"
                                    "The ASRL keyword is used for serial interface.\n"
                                    "The GPIB keyword is used for GPIB interface.\n"
                                    "The RSIB keyword is used for RSIB interface.\n\n"
                                    "Examples:\n"
                                    "(1) Serial - ASRL1\n"
                                    "(2) GPIB   - GPIB::20\n"
                                    "(3) RSIB   - RSIB::192.168.1.33::INSTR\n\n"
                                    "The default value for optional parameters are shown below.\n\n"
                                    "Optional Parameter          Default Value\n"
                                    "-----------------------------------------\n"
                                    "board                       0\n"
                                    "secondary address           none - 31\n\n"
                                    "Note:\n\n"
                                    "(1) Parameters of the serial interface, you must set via T&M\n"
                                    "Expolrer before you call initialize function. Default values are\n"
                                    "Baud Rate: 9600, Data Bits: 8, Parity: None, Stop Bits: 1 and\n"
                                    "Flow Control: None"
                                    };  
        
static char *configCtrlsHelpStr[] = {
                                    "This control sets the start frequency value.\n\n"
                                    "Valid Range:\n"
                                    "models ZVR, ZVRL, ZVRE: 9.0e3 to 4.0e9\n"
                                    "models ZVC, ZVCE:      20.0e3 to 8.0e9\n"
                                    "models ZVM:            10.0e6 to 21.0e9\n"
                                    "models ZVK:            10.0e6 to 40.0e9\n\n"
                                    "Default Value: 2.17e9 Hz",
                                    
                                    "This control sets the stop frequency value.\n\n"
                                    "models ZVR, ZVRL, ZVRE: 9.0e3 to 4.0e9\n"
                                    "models ZVC, ZVCE:      20.0e3 to 8.0e9\n"
                                    "models ZVM:            10.0e6 to 21.0e9\n"
                                    "models ZVK:            10.0e6 to 40.0e9\n\n"
                                    "Default Value: 2.27e9 Hz",
                                    
                                    "This control sets the format of measurement.\n\n"
                                    "Valid Range:\n"
                                    "0 - Magnitude (Default Value)\n"
                                    "1 - Phase"};
static char *measCtrlsHelpStr[] = { "This Graph displays the data read from the instrument"};
                                    
static char backgroundPnlHelpStr[] = {
                                    "This function illustrates typical measuring task for a network\n"
                                    "analyzer. The described measured task is transmission\n"
                                    "measurement of magnitude and phase.\n\n"
                                    "Note:\n\n"
                                    "(1) Connect the input of the DUT to PORT1 and the output\n"
                                    "to PORT 2 on the ZVx.\n\n"};
                     
static char initPnlHelpStr[] =     {"This panel performs the following initialization actions:\n\n"
                                    "- Opens a session to the Default Resource Manager resource and a\n"
                                    "session to the specified device using the interface and address\n"
                                    "specified in the Resource_Name control.\n\n"
                                    "- Performs an identification query on the Instrument.\n\n"
                                    "- Resets the instrument to a known state.\n\n"
                                    "- Sends initialization commands to the instrument that set any\n"
                                    "necessary programmatic variables such as Headers Off, Short\n"
                                    "Command form, and Data Transfer Binary to the state necessary\n"
                                    "for the operation of the instrument driver.\n\n"
                                    "- Returns an Instrument Handle which is used to differentiate\n"
                                    "between different sessions of this instrument driver.\n\n"
                                    "- Each time this function is invoked a Unique Session is opened.\n"
                                    "It is possible to have more than one session open for the same\n"
                                    "resource."};

static char configPnlHelpStr[] = {"This panel configures the instrument."
};

static char measPnlHelpStr[] =   {"\nThis panel displays the measured values.\n"
                                    "Configuration of the instrument is not\n"
                                    "possible with this panel. Press the\n"
                                    "\"Configure\" button to return to\n"
                                    "the configuration panel."};

/*= Main Function ===========================================================*/

void main ()
{
    /* Load panels.*/
    panelHandle[BCKGRND] = LoadPanel (panelHandle[BCKGRND], "rszvu.uir", BCKGRND);
    panelHandle[MEAS] = LoadPanel (panelHandle[BCKGRND], "rszvu.uir", MEAS);
    panelHandle[CONFIG] = LoadPanel (panelHandle[BCKGRND], "rszvu.uir", CONFIG); 
    panelHandle[INIT] = LoadPanel (panelHandle[BCKGRND], "rszvu.uir", INIT);
    /* Display panels.*/
    DisplayPanel (panelHandle[BCKGRND]);
    DisplayPanel (panelHandle[INIT]);
    uirCode = RunUserInterface ();
}

/*===========================================================================*/
/* Function: Initialize Instrument                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Initialize panel. It initializes the instrument and switches to */
/*           the panel Configure.                                            */
/*===========================================================================*/
int CVICALLBACK initInstrument (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
     
    ViChar instrDescr[256], errorMessage[256], errBuffer[256];
    ViInt16 response, addr, rst, id;
    int interface;

    switch (event)
        {
        case EVENT_COMMIT:
            SetWaitCursor(0);
            GetCtrlVal (panelHandle[INIT], INIT_RSRC_NAME, instrDescr);
            GetCtrlVal (panelHandle[INIT], INIT_ID, &id);
            GetCtrlVal (panelHandle[INIT], INIT_RST, &rst);
            
            SetWaitCursor(1);
            if ((uirErr = rszv_init (instrDescr, id, rst, &instrHandle)) < 0 )
                {
                rszv_error_message(VI_NULL, uirErr, errorMessage);
                Fmt (errBuffer, "%s<Initialization Error:\n\n%s\n\nCheck your "
                        "connections and make sure you have defined right interface descriptor.",
                        errorMessage);
                MessagePopup ("ERROR!", errBuffer);
                SetWaitCursor(0);
                return(0);   
                }
            SetWaitCursor(0);
            HidePanel (panelHandle[INIT]);
            DisplayPanel (panelHandle[CONFIG]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button causes the instrument to be initialized.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Configure Measurement                                           */
/* Purpose:  This is a callback function of the Continue button on the       */
/*           Configure panel (configure & measure).                          */
/*===========================================================================*/
int CVICALLBACK configMeasurement (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char errMsg[256];
    int i, format, count = 401;
    double mkrFrq, mkrLevel, startFrq, stopFrq;
    static double data[MAX_ARRAY];

    switch (event) {
        case EVENT_COMMIT:
            SetWaitCursor(0);
            HidePanel (panelHandle[CONFIG]);
            DisplayPanel (panelHandle[MEAS]);
            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_IMMEDIATE_DRAW);
            SetWaitCursor(1);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_START, &startFrq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_STOP, &stopFrq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_FORMAT, &format);

            APPCHECKERR( rszv_appExample (instrHandle, startFrq, stopFrq, format, &count, data));

            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_IMMEDIATE_DRAW);
            PlotWaveform (panelHandle[MEAS], MEAS_GRAPH, data, count,
                        VAL_DOUBLE, 1.0, 0.0, 0.0, 1.0,
                        VAL_CONNECTED_POINTS, VAL_NO_POINT, VAL_SOLID, 1,
                        VAL_YELLOW);

            SetCtrlVal (panelHandle[MEAS], MEAS_LOW, startFrq);
            SetCtrlVal (panelHandle[MEAS], MEAS_HIGH, stopFrq);
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will configure the instrument to take a measurement.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Take Measurement                                                */
/* Purpose:  This is a callback function of the button Measure on the        */
/*           panel Measure. It returns a measurement value without           */
/*           reconfiguring of the instrument.                                */
/*===========================================================================*/
int CVICALLBACK takeMeasurement(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{  
    long error;
    double startFrq, stopFrq;
    char errMsg[256];
    int i, format, count = 401;
    static double data[MAX_ARRAY];

    switch (event) {
        case EVENT_COMMIT:
            SetWaitCursor(1);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_START, &startFrq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_STOP, &stopFrq);
            GetCtrlVal (panelHandle[CONFIG], CONFIG_FORMAT, &format);

            APPCHECKERR( rszv_actSweepStart (instrHandle));
            APPCHECKERR( rszv_dataReadTrace(instrHandle, VI_FALSE, VI_FALSE, 0, 0, &count, data));

            DeleteGraphPlot (panelHandle[MEAS], MEAS_GRAPH, -1, VAL_IMMEDIATE_DRAW);
            PlotWaveform (panelHandle[MEAS], MEAS_GRAPH, data, count,
                            VAL_DOUBLE, 1.0, 0.0, 0.0, 1.0, VAL_CONNECTED_POINTS, VAL_NO_POINT, 
                            VAL_SOLID, 1, VAL_YELLOW);

            SetCtrlVal (panelHandle[MEAS], MEAS_LOW, startFrq);
            SetCtrlVal (panelHandle[MEAS], MEAS_HIGH, stopFrq);
            SetWaitCursor(0);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will return the new meaurement value.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Cancel                                                          */
/* Purpose:  Called by the init panel this function pops up a confirmation   */
/*           dialog box and then quits the user interface, if desired.       */
/*===========================================================================*/
int CVICALLBACK Cancel (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", 
                   "Are you sure you want to quit this application?")) == 1) {
                QuitUserInterface (uirCode);
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Control Help", "Closes the application.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Control Help                                                    */
/* Purpose:  This is a callback function of all controls that configure the  */
/*           instrument. On the right mouse-click on the control a help      */
/*           window describing its purpose is displayed.                     */
/*===========================================================================*/
int CVICALLBACK controlHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{   
    SetWaitCursor(0);
    if (event == EVENT_RIGHT_CLICK) {
        if (panel == panelHandle[INIT]) 
            MessagePopup ("Help", initCtrlsHelpStr[control-4]);
        if (panel == panelHandle[CONFIG]) 
            MessagePopup ("Help", configCtrlsHelpStr[control-4]);
        if (panel == panelHandle[MEAS]) 
            MessagePopup ("Help", measCtrlsHelpStr[control-6]);
    }
    return 0;
}

/*===========================================================================*/
/* Function: Launch Configure Panel                                          */
/* Purpose:  This is a callback function of the button Configure on the      */
/*           panel Measure. It returns back to the Configuration panel to be */
/*           able to change configuration without having to re-run this      */
/*           program.                                                        */
/*===========================================================================*/
int CVICALLBACK launchConfig (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            HidePanel (panelHandle[MEAS]);
            DisplayPanel (panelHandle[CONFIG]);
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will recall the configuration panel.");
            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    SetWaitCursor(0);
    if (panel == panelHandle[BCKGRND])
        MessagePopup ("Help", backgroundPnlHelpStr);
    if (panel == panelHandle[INIT])
        MessagePopup ("Help", initPnlHelpStr);
    if (panel == panelHandle[CONFIG])
        MessagePopup ("Help", configPnlHelpStr);
    if (panel == panelHandle[MEAS]) {
        MessagePopup ("Help", measPnlHelpStr);
    }    
    return;
}

/*===========================================================================*/
/* Function: Launch Close Panel                                              */
/* Purpose:  This is a callback function of the button Close on the          */
/*           panel Measure. It pops-up a panel that will close the           */
/*           instrument.                                                     */
/*===========================================================================*/
int CVICALLBACK launchClose (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;
    
    SetWaitCursor(0);
    switch (event) {
        case EVENT_COMMIT:
            if ((ConfirmPopup ("Exit Application", "Are you sure you want to quit "
                    "this application?")) == 1) {
                if ((uirErr = rszv_close (instrHandle)) < 0) {
                    rszv_error_message(VI_NULL, uirErr, errorMessage);
                    Fmt (errBuffer, "%s<Application failed to close the "
                        "instrument.\n\n%s\n\nExiting Application.", errorMessage);
                    MessagePopup("ERROR!", errBuffer);
                }
                QuitUserInterface (uirCode);
                exit (-1);
            }
            break;
        case EVENT_RIGHT_CLICK:
            MessagePopup ("Help","This button will pop-up a panel to close the instrument.");
            break;
    }
    return 0;
}   

/*****************************************************************************/
long abortApplication (void)
{
    ViChar errorMessage[256], errBuffer[256];
    ViInt16 response;

    SetWaitCursor (0);
    rszv_error_message(VI_NULL, uirErr, errorMessage);
    Fmt (errBuffer,"%s<The following error caused the application to fail:\n\n%s\n\n"
        "Attempting to Close Instrument Driver and Exit Application", 
        errorMessage);
    MessagePopup("ERROR!, Application Failed", errBuffer);
    
    if ((uirErr = rszv_close (instrHandle)) < 0)
        {
        rszv_error_message(VI_NULL, uirErr, errorMessage);
        Fmt (errBuffer, "%s<Application failed to close the Instrument Driver.\n\n"
            "%s\n\nExiting Application.", errorMessage);
        MessagePopup("ERROR!",errBuffer);
        }
    else 
        MessagePopup("Application Aborted","Instrument Driver Closed, Exiting Application.");
        
    QuitUserInterface (uirCode);
    exit (-1);
    
    return 0;
}

/*= End =====================================================================*/

